﻿using UnityEngine;
using System.Collections;

public class EnemyHealth : MonoBehaviour {

	public GameObject healthBar;
	private GameObject healthBarInst;
	public GameObject healthDrop;
	private GameObject healthDropInst;
	private float offSet;
	private EnemyHealthBar ebh;
	public int health;
	private int startingHealth;	//the amount of health an enemy has before he starts taking damage;
	/*
		1 = Fire
		2 = Electricity
		3 = Frailty
	*/
	public int weakness;	//what this enemy is weak to
	public bool burning;	//are we on fire?
	public bool electric;	//are we shocking?
	public bool flung;
	public bool dead;		//are we we are the waiting unknown?
	private Renderer mr;
	private BasicEnemyMovement bem;
	private RangedEnemyMovement rem;

	float lastFireDamage = -10f;
	float deltaTime = 0f;
	float delay = 1f;	//interval in between time damage is dealt
	float fireTime;		//how long the enemy remains on fire
	int fireDamage;		//the amount of damage dealt per second when enemy is on fire

	//woogie woogie woogie
	float lastElectricDamage = -10f;
	float electricTime;
	int electricDamage;

	float flingForce;
	float flingStunTime;
	bool fireOnce = true;
	// Use this for initialization
	protected void Awake () {
		startingHealth = health;
		if (GetComponent<BasicEnemyMovement>() != null) {
			bem = GetComponent<BasicEnemyMovement>();
		}

		if (GetComponent<RangedEnemyMovement>() != null) {
			rem = GetComponent<RangedEnemyMovement>();
		}

		dead = false;
		burning = false;
		electric = false;
		mr = GetComponent<Renderer>();

		//give the enemy a health bar
		healthBarInst = Instantiate(healthBar) as GameObject;
		healthBarInst.transform.SetParent(GameObject.FindGameObjectWithTag("PlayerUI").transform);
		ebh = healthBarInst.GetComponent<EnemyHealthBar>();
		ebh.eh = this;
		ebh.xScale = 0.2f;
		ebh.yScale = 0.05f;
		offSet = Screen.height*0.15f;

		//set status effect damage and duration
		fireTime = (float)(startingHealth/10f);
		fireDamage = startingHealth/30;

		electricTime = (float)(startingHealth/15f);
		electricDamage = startingHealth/40;

		flingForce = (float)(startingHealth/10f);
		flingStunTime = (float)(startingHealth/15f);

		//Debug.Log("Burn stats: "+fireTime+"||"+fireDamage);
		//Debug.Log("Shock stats: "+electricTime+"||"+electricDamage);
	}
	
	// Update is called once per frame
	protected void Update () {
		//set healthbar position
		Vector3 screenPos = Camera.main.WorldToScreenPoint(transform.position);
		ebh.xPos = screenPos.x;
		ebh.yPos = screenPos.y+offSet;

		if (burning) {
			OnFire();
		}

		if (electric) {
			Electrocuted();
		}

		if (flung) {
			Flung();
		}

		//Debug.Log("Enemy is burning: "+burning);

		if (health <= 0) {
			if (bem != null) {
				bem.enabled = false;
			}
			if (rigidbody != null) {
				rigidbody.isKinematic = true;
			}

			healthDropInst = Instantiate(healthDrop, transform.position, transform.rotation) as GameObject;
			healthDropInst.GetComponent<HealthPickup>().restore = startingHealth/5;

			dead = true;
			Destroy(gameObject);
			this.enabled = false;
		}
	}

	public void Hurt(int damage) {
		Debug.Log("hurt "+ damage);
		health -= damage;
		StartCoroutine(Hurting());
	}

	IEnumerator Hurting() {
		mr.material = null;
		yield return new WaitForSeconds(0.5f);
		mr.material.color = Color.white;
	}
	
	public void OnFire() {
		deltaTime += Time.deltaTime;	//start incrementing the timer
		//Debug.Log("TOASTY for: "+deltaTime);
		if (deltaTime <= fireTime) {
			//This next section deals damage during the time the enemy is on fire, but only during every second.
			if (deltaTime > delay+lastFireDamage) {
				Hurt(fireDamage);		//deal the damage
				Debug.Log(gameObject.name+" is burning");
				lastFireDamage = deltaTime;	//set the variable to the time the damage was dealt so the loop continues
			}
		}
		else {
			deltaTime = 0;		//reset the timer
			burning = false;	//we are no longer burning
		}
	}

	//for when things get SHOCKING
	public void Electrocuted() {
		deltaTime += Time.deltaTime;
		if (deltaTime <= electricTime) {
			if (bem != null) {
				bem.stunned = true;
			}
			if (rem != null) {
				rem.stunned = true;
			}
			if (deltaTime > delay+lastElectricDamage) {
				Hurt(electricDamage);
				Debug.Log(gameObject.name+" is SHOCKED at this predicament");
				lastElectricDamage = deltaTime;	//set the variable to the time the damage was dealt so the loop continues
			}
		}
		else {	//otherwise, reset everything
			deltaTime = 0;
			electric = false;
			if (bem != null) {
				bem.stunned = false;
			}
			if (rem != null) {
				rem.stunned = false;
			}
		}
	}

	//knocks the enemy back and stuns it
	public void Flung() {
		if (fireOnce) {
			Vector3 player = GameObject.FindGameObjectWithTag("Player").transform.position;
			Vector3 direction = transform.position-player;
			rigidbody.AddForce(direction*flingForce, ForceMode.Impulse);
		}
		deltaTime += Time.deltaTime;

		if (deltaTime > 0) {
			fireOnce = false;
		}

		if (deltaTime <= flingStunTime) {
			if (bem != null) {
				bem.stunned = true;
			}
			if (rem != null) {
				rem.stunned = true;
			}
		}
		else {
			flung = false;
			fireOnce = true;
			if (bem != null) {
				bem.stunned = false;
			}
			if (rem != null) {
				rem.stunned = false;
			}
		}
	}

	IEnumerator Delay(float time) {
		yield return new WaitForSeconds(time);
		//attackCooldown = false;
	}
}
